@@ -99,21 +99,28 @@ static gboolean		mapi_folder_synchronize_sync
 G_DEFINE_TYPE (CamelMapiFolder, camel_mapi_folder, CAMEL_TYPE_OFFLINE_FOLDER)
 
 static GPtrArray *
-mapi_folder_search_by_expression (CamelFolder *folder, const gchar *expression, GError **error)
+mapi_folder_search_by_expression (CamelFolder *folder,
+				  const gchar *expression,
+				  GCancellable *cancellable,
+				  GError **error)
 {
 	CamelMapiFolder *mapi_folder = CAMEL_MAPI_FOLDER(folder);
 	GPtrArray *matches;
 
 	CAMEL_MAPI_FOLDER_LOCK(mapi_folder, search_lock);
 	camel_folder_search_set_folder (mapi_folder->search, folder);
-	matches = camel_folder_search_search(mapi_folder->search, expression, NULL, error);
+	matches = camel_folder_search_search(mapi_folder->search, expression, NULL, cancellable, error);
 	CAMEL_MAPI_FOLDER_UNLOCK(mapi_folder, search_lock);
 
 	return matches;
 }
 
 static GPtrArray *
-mapi_folder_search_by_uids (CamelFolder *folder, const gchar *expression, GPtrArray *uids, GError **error)
+mapi_folder_search_by_uids (CamelFolder *folder,
+			    const gchar *expression,
+			    GPtrArray *uids,
+			    GCancellable *cancellable,
+			    GError **error)
 {
 	GPtrArray *matches;
 	CamelMapiFolder *mapi_folder = CAMEL_MAPI_FOLDER (folder);
@@ -123,7 +130,7 @@ mapi_folder_search_by_uids (CamelFolder *folder, const gchar *expression, GPtrAr
 
 	CAMEL_MAPI_FOLDER_LOCK (mapi_folder, search_lock);
 	camel_folder_search_set_folder (mapi_folder->search, folder);
-	matches = camel_folder_search_search (mapi_folder->search, expression, uids, error);
+	matches = camel_folder_search_search (mapi_folder->search, expression, uids, cancellable, error);
 	CAMEL_MAPI_FOLDER_UNLOCK (mapi_folder, search_lock);
 
 	return matches;
@@ -613,6 +620,7 @@ gather_object_summary_cb (EMapiConnection *conn,
 			const gchar *subject, *message_id, *references, *in_reply_to, *display_to, *display_cc;
 			const struct FILETIME *delivery_time, *submit_time;
 			const uint32_t *msg_size;
+			gint offset = 0;
 			gchar *formatted_addr, *from_name, *from_email;
 			CamelAddress *to_addr, *cc_addr, *bcc_addr;
 
@@ -636,9 +644,9 @@ gather_object_summary_cb (EMapiConnection *conn,
 			minfo->info.size = msg_size ? *msg_size : 0;
 
 			if (minfo->info.date_sent != 0)
-				minfo->info.date_sent = minfo->info.date_sent;
+				minfo->info.date_sent = camel_header_decode_date (ctime (&minfo->info.date_sent), &offset);
 			if (minfo->info.date_received != 0)
-				minfo->info.date_received = minfo->info.date_received;
+				minfo->info.date_received = camel_header_decode_date (ctime (&minfo->info.date_received), &offset);
 
 			/* Threading related properties */
 			mapi_set_message_id (minfo, message_id);
@@ -731,7 +739,7 @@ gather_object_summary_cb (EMapiConnection *conn,
 gboolean
 camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable, GError **mapi_error)
 {
-	gboolean status, has_obj_folder;
+	gboolean status;
 	gboolean full_download;
 	CamelSettings *settings;
 	CamelStore *store = camel_folder_get_parent_store (folder);
@@ -772,7 +780,6 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 	}
 
 	status = cmf_open_folder (mapi_folder, conn, &obj_folder, cancellable, mapi_error);
-	has_obj_folder = status;
 
 	if (status) {
 		status = e_mapi_connection_get_folder_properties (conn, &obj_folder, NULL, NULL, e_mapi_utils_get_folder_basic_properties_cb, &fbp, cancellable, mapi_error);
@@ -850,8 +857,7 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 		camel_folder_change_info_free (gos.changes);
 	}
 
-	if (has_obj_folder)
-		e_mapi_connection_close_folder (conn, &obj_folder, cancellable, mapi_error);
+	e_mapi_connection_close_folder (conn, &obj_folder, cancellable, mapi_error);
 
 	g_slist_free_full (gco.to_update, g_free);
 	if (gco.removed_uids)
