@@ -41,7 +41,6 @@
 
 #include "camel-mapi-store.h"
 #include "camel-mapi-folder.h"
-#include "camel-mapi-sasl-krb.h"
 #include "camel-mapi-settings.h"
 #include "camel-mapi-store-summary.h"
 #include "camel-mapi-folder-summary.h"
@@ -819,6 +818,11 @@ mapi_get_folder_info_offline (CamelStore *store, const gchar *top,
 
 	unescape_folder_names (fi);
 
+	if (!fi && error && !*error)
+		g_set_error_literal (error, CAMEL_STORE_ERROR, CAMEL_STORE_ERROR_NO_FOLDER,
+			(flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIPTION_LIST) != 0 ?
+			_("No public folder found") : _("No folder found"));
+
 	return fi;
 }
 
@@ -1257,7 +1261,7 @@ mapi_store_create_folder_sync (CamelStore *store,
 		g_set_error_literal (
 			error, CAMEL_SERVICE_ERROR,
 			CAMEL_SERVICE_ERROR_UNAVAILABLE,
-			_("MAPI folders can be created only within user's mailbox"));
+			_("MAPI folders can be created only within mailbox of the logged user"));
 		return NULL;
 	}
 
@@ -1973,9 +1977,6 @@ camel_mapi_store_class_init (CamelMapiStoreClass *class)
 	CamelServiceClass *service_class;
 	CamelStoreClass *store_class;
 
-	/* register MAPIKRB auth type */
-	CAMEL_TYPE_MAPI_SASL_KRB;
-
 	g_type_class_add_private (class, sizeof (CamelMapiStorePrivate));
 
 	object_class = G_OBJECT_CLASS (class);
@@ -2105,7 +2106,6 @@ mapi_connect_sync (CamelService *service,
 	CamelMapiStore *store = CAMEL_MAPI_STORE (service);
 	CamelServiceConnectionStatus status;
 	CamelSession *session;
-	EMapiProfileData empd = { 0 };
 	uint64_t current_size = -1, receive_quota = -1, send_quota = -1;
 	gchar *name;
 
@@ -2132,9 +2132,7 @@ mapi_connect_sync (CamelService *service,
 	name = camel_service_get_name (service, TRUE);
 	camel_operation_push_message (cancellable, _("Connecting to '%s'"), name);
 
-	e_mapi_util_profiledata_from_settings (&empd, CAMEL_MAPI_SETTINGS (camel_service_get_settings (service)));
-
-	if (!camel_session_authenticate_sync (session, service, empd.krb_sso ? "MAPIKRB" : NULL, cancellable, error)) {
+	if (!camel_session_authenticate_sync (session, service, NULL, cancellable, error)) {
 		camel_operation_pop_message (cancellable);
 		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		camel_service_disconnect_sync (service, TRUE, NULL);
@@ -2566,20 +2564,20 @@ mapi_authenticate_sync (CamelService *service,
 	profile = camel_mapi_settings_get_profile (mapi_settings);
 
 	if (empd.krb_sso) {
-		if (!e_mapi_util_trigger_krb_auth (&empd, error))
+		if (e_mapi_util_trigger_krb_auth (&empd, error))
+			return CAMEL_AUTHENTICATION_ACCEPTED;
+		else
 			return CAMEL_AUTHENTICATION_ERROR;
+	}
 
-		password = NULL;
-	} else {
-		password = camel_service_get_password (service);
+	password = camel_service_get_password (service);
 
-		if (password == NULL) {
-			g_set_error_literal (
-				error, CAMEL_SERVICE_ERROR,
-				CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
-				_("Authentication password not available"));
-			return CAMEL_AUTHENTICATION_ERROR;
-		}
+	if (password == NULL) {
+		g_set_error_literal (
+			error, CAMEL_SERVICE_ERROR,
+			CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+			_("Authentication password not available"));
+		return CAMEL_AUTHENTICATION_ERROR;
 	}
 
 	store->priv->conn = e_mapi_connection_new (profile, password, cancellable, &mapi_error);
@@ -2601,7 +2599,6 @@ mapi_authenticate_sync (CamelService *service,
 			error, CAMEL_SERVICE_ERROR,
 			CAMEL_SERVICE_ERROR_UNAVAILABLE,
 			mapi_error->message);
-		g_clear_error (&mapi_error);
 		result = CAMEL_AUTHENTICATION_ERROR;
 	} else {
 		/* mapi_error should be set */
